#include <Python.h>
#include "cmdsetprop.h"
#include "cmdutil.h"
#include "cmdvar.h"

PyObject *scribus_setgradfill(PyObject *self, PyObject* args)
{
	char *Name = "";
	char *Color1;
	char *Color2;
	int typ, shade1, shade2;
	if (!PyArg_ParseTuple(args, "isisi|s", &typ, &Color1, &shade1, &Color2, &shade2, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetGradientFill(type, color1, shade1, color2, shade2 [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *b = GetUniqueItem(QString(Name));
	if (b != NULL)
	{
		QColor tmp;
		b->fill_gradient.clearStops();
		QString c1 = QString(Color1);
		QString c2 = QString(Color2);
		b->SetFarbe(&tmp, c1, shade1);
		b->fill_gradient.addStop(tmp, 0.0, 0.5, 1.0, c1, shade1);
		b->SetFarbe(&tmp, c2, shade2);
		b->fill_gradient.addStop(tmp, 1.0, 0.5, 1.0, c2, shade2);
		b->GrType = typ;
		b->OwnPage->updateGradientVectors(b);
		b->OwnPage->RefreshItem(b);
	}
	return Py_None;
}

PyObject *scribus_setfillcolor(PyObject *self, PyObject* args)
{
	char *Name = "";
	char *Color;
	if (!PyArg_ParseTuple(args, "s|s", &Color, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetFillColor(color [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->Pcolor = QString(Color);
	return Py_None;
}

PyObject *scribus_setlinecolor(PyObject *self, PyObject* args)
{
	char *Name = "";
	char *Color;
	if (!PyArg_ParseTuple(args, "s|s", &Color, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineColor(color [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *it = GetUniqueItem(QString(Name));
	if (it != NULL)
		it->Pcolor2 = QString(Color);
	return Py_None;
}

PyObject *scribus_setlinewidth(PyObject *self, PyObject* args)
{
	char *Name = "";
	double w;
	if (!PyArg_ParseTuple(args, "d|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineWidth(width [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if ((!Carrier->HaveDoc) || ((w < 0.0) || (w > 12.0)))
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->Pwidth = w;
	return Py_None;
}

PyObject *scribus_setlineshade(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineShade(shade [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if ((!Carrier->HaveDoc) || ((w < 0) || (w > 100)))
		return Py_None;
	PageItem *it = GetUniqueItem(QString(Name));
	if (it != NULL)
		it->Shade2 = w;
	return Py_None;
}

PyObject *scribus_setfillshade(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetFillShade(shade [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if ((!Carrier->HaveDoc) || ((w < 0) || (w > 100)))
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->Shade = w;
	return Py_None;
}

PyObject *scribus_setlinejoin(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineJoin(jointype [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->PLineJoin = Qt::PenJoinStyle(w);
	return Py_None;
}

PyObject *scribus_setlineend(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineEnd(endtype [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->PLineEnd = Qt::PenCapStyle(w);
	return Py_None;
}

PyObject *scribus_setlinestyle(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetLineStyle(style [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *i = GetUniqueItem(QString(Name));
	if (i != NULL)
		i->PLineArt = Qt::PenStyle(w);
	return Py_None;
}

PyObject *scribus_setcornerrad(PyObject *self, PyObject* args)
{
	char *Name = "";
	int w;
	if (!PyArg_ParseTuple(args, "i|s", &w, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetCornerRadius(radius [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if ((!Carrier->HaveDoc) || (w < 0))
		return Py_None;
	PageItem *b = GetUniqueItem(QString(Name));
	if (b != NULL)
	{
		if ((b->PType == 2) || (b->PType == 3) || (b->PType == 4))
		{
			b->RadRect = w;
			if (w > 0)
				b->OwnPage->SetFrameRound(b);
		}
		else
				b->OwnPage->SetRectFrame(b);
	}
	return Py_None;
}

PyObject *scribus_setmultiline(PyObject *self, PyObject* args)
{
	char *Name = "";
	char *Color;
	if (!PyArg_ParseTuple(args, "s|s", &Color, &Name))
	{
		PyErr_SetString(PyExc_Exception, ERRPARAM + QString("SetMultiLine(color [, objectname])"));
		return NULL;
	}
	Py_INCREF(Py_None);
	if (!Carrier->HaveDoc)
		return Py_None;
	PageItem *b = GetUniqueItem(QString(Name));
	if (b != NULL)
		if (Carrier->doc->MLineStyles.contains(QString(Color)))
			b->NamedLStyle = QString(Color);
	return Py_None;
}
